from itertools import chain
from typing import Generator

from airfold_common.format import ChFormat, Format
from airfold_common.project import ProjectFile
from cachetools import TTLCache, cached

from airfold_cli.api import AirfoldApi


@cached(cache=TTLCache(maxsize=10000, ttl=5))
def get_source_names() -> list[str]:
    api = AirfoldApi.from_config()
    files = api.pull()
    formatter: Format = ChFormat()
    sources: list[ProjectFile] = list(filter(lambda f: formatter.is_source(f.data), files))
    return [source.name for source in sources]


@cached(cache=TTLCache(maxsize=10000, ttl=5))
def get_importable_tables() -> dict[str, list[str]]:
    api = AirfoldApi.from_config()
    return api.get_importable_tables()


def source_name_completion(cur: str) -> Generator[str, None, None]:
    """Pipe name completion."""
    try:
        source_names: list[str] = get_source_names()
        yield from filter(lambda name: name.startswith(cur), source_names) if cur else source_names
    except Exception as e:
        pass


def importable_table_name_completion(cur: str) -> Generator[str, None, None]:
    """Importable table name completion."""
    try:
        importable_tables: dict[str, list[str]] = get_importable_tables()
        table_names: set[str] = set(chain(*importable_tables.values()))
        yield from filter(lambda name: name.startswith(cur), table_names) if cur else table_names
    except Exception as e:
        pass


def importable_database_name_completion(cur: str) -> Generator[str, None, None]:
    """Importable database name completion."""
    try:
        importable_tables: dict[str, list[str]] = get_importable_tables()
        db_names: list[str] = list(importable_tables.keys())
        yield from filter(lambda name: name.startswith(cur), db_names) if cur else db_names
    except Exception as e:
        pass
